/**
 * 
 */
package lm.parser.dynamic;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

import lm.parser.dynamic.syntax.Syntax;
import lm.parser.dynamic.syntax.SyntaxException;

import org.apache.log4j.Logger;


/**
 * @author Michal Jaros
 *
 */
public class Parser implements ActionListener, DocumentListener {

	/**
	 * Skadnia (gramatyka)
	 */
	private Syntax syntax = null;

	/**
	 * Utworzenie skladni (dynamicznych struktur danych) dla zadanej gramatyki
	 * @param grama - gramatyka podana przez uzytkownika
	 */
	private void compileGrama(String grama) {
		try {
			syntax = new Syntax(grama);
		} catch (SyntaxException e) {
			syntax = null;
			log("Bdnia gramatyka !!!", e);
		}
		if (syntax != null) {
			log("Gramatyka gotowa do uycia");
		}
	}

	/**
	 * Analiza zdania
	 * @param sentence - zdanie do analizy
	 */
	private void parseSentence(String sentence) {
		if (syntax.parse("S", sentence)) {
			log("Zdanie poprawne");
		} else {
			log("Zdanie niepoprawne !!!");
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new Parser().show();
		//new Syntax("  ala	\r\n= B        B \"12\"\tB .  B    =	\n\"3\". C=B .           ");
		//Syntax syn = new Syntax("S=A B|B.\nA=\"Ala\"|\"Kot\"|\"Diablo\".\nB=\"hehehe\".");
		//syn.parse("Kot   hehehe");
		//Syntax syn = new Syntax("S=W\";\"Z.Z=W\";\"Z|\"\".W=P Wp.Wp=O W|\"\".P=L|\"(\"W\")\".L=C Lp.Lp=L|\"\".C=\"0\"|\"1\"|\"2\"|\"3\"|\"4\"|\"5\"|\"6\"|\"7\"|\"8\"|\"9\".O=\"*\"|\":\"|\"+\"|\"-\"|\"^\".");
		//syn.parse("(12*3)+5-(234+3)^3;12+3;");
	}

	/**
	 * Logger
	 */
	private static final Logger log = Logger.getLogger(Parser.class);
	
	/**
	 * Pokazanie okna
	 */
	public void show(){
		log.debug("Show");
		getWindowFrame().setVisible(true);
	}
	
	/**
	 * Komenda dla kompilacji gramatyki
	 */
	private static final String COMMAND_COMPILE = "compile";

	/**
	 * Komenda dla analizy zdania
	 */
	private static final String COMMAND_PARSE = "parse";

	/**
	 * Czcionka
	 */
	private static final Font font = new Font(null, Font.PLAIN, 16);
	
	/**
	 * Okno
	 */
	private JFrame windowFrame = null;
	
	/**
	 * Zawartosc okna
	 */
	private JPanel windowContent = null;

	/**
	 * Obszar z gramatyka
	 */
	private JPanel gramaPanel = null;

	/**
	 * Gramatyka
	 */
	private JTextArea grama = null;

	/**
	 * Przycisk przetwarzajacy gramatyke
	 */
	private JButton compileButton = null;

	/**
	 * Obszar z zdaniem
	 */
	private JPanel sentencePanel = null;

	/**
	 * Zdanie do analizy
	 */
	private JTextArea sentence = null;
	
	/**
	 * Przycisk sprawdzajacy zdanie
	 */
	private JButton parseButton = null;

	/**
	 * Obszar z logami
	 */
	private JPanel logPanel = null;

	/**
	 * Log
	 */
	private JTextArea logText = null;
	
	/**
	 * @return the windowFrame
	 */
	private JFrame getWindowFrame() {
		if (windowFrame == null) {
			log.debug("create windowFrame");
			JFrame.setDefaultLookAndFeelDecorated(true);
			windowFrame = new JFrame();
			windowFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			windowFrame.setTitle("Analizator skadniowy sterowany skadni");
			windowFrame.setContentPane(getWindowContent());
			windowFrame.pack();
		}
		return windowFrame;
	}

	/**
	 * @return the windowContent
	 */
	private JPanel getWindowContent() {
		if (windowContent == null) {
			log.debug("Create windowContent");
			windowContent = new JPanel();
			windowContent.setLayout(new BoxLayout(windowContent, BoxLayout.Y_AXIS));
			JSplitPane topSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
			topSplitPane.setLeftComponent(getGramaPanel());
			topSplitPane.setRightComponent(getSentencePanel());
			topSplitPane.setOneTouchExpandable(true);
			JSplitPane mainSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			mainSplitPane.setTopComponent(topSplitPane);
			mainSplitPane.setBottomComponent(getLogPanel());
			mainSplitPane.setOneTouchExpandable(true);
			windowContent.add(mainSplitPane);
		}
		return windowContent;
	}

	/**
	 * @return
	 */
	private JPanel getGramaPanel(){
		if (gramaPanel == null) {
			log.debug("create grama panel");
			gramaPanel = new JPanel();
			gramaPanel.setLayout(new BoxLayout(gramaPanel, BoxLayout.Y_AXIS));
			JLabel label = new JLabel("Gramatyka");
			label.setFont(font);
			gramaPanel.add(label);
			JScrollPane scrollPane = new JScrollPane(getGrama());
			scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			scrollPane.setPreferredSize(new Dimension(350, 250));
			gramaPanel.add(scrollPane);
			gramaPanel.add(getCompileButton());
		}
		return gramaPanel;
	}

	/**
	 * @return
	 */
	private JTextArea getGrama() {
		if (grama == null) {
			log.debug("create grama");
			grama = new JTextArea();
			grama.setLineWrap(true);
			grama.setWrapStyleWord(true);
			grama.setFont(font);
			grama.getDocument().addDocumentListener(this);
		}
		return grama;
	}

	/**
	 * @return
	 */
	private JButton getCompileButton() {
		if (compileButton == null) {
			log.debug("create compile button");
			compileButton = new JButton("Kompiluj");
			compileButton.setEnabled(false);
			compileButton.setActionCommand(COMMAND_COMPILE);
			compileButton.addActionListener(this);
		}
		return compileButton;
	}

	/**
	 * @return
	 */
	private JPanel getSentencePanel(){
		if (sentencePanel == null) {
			log.debug("create sentence panel");
			sentencePanel = new JPanel();
			sentencePanel.setLayout(new BoxLayout(sentencePanel, BoxLayout.Y_AXIS));
			JLabel label = new JLabel("Zdanie");
			label.setFont(font);
			sentencePanel.add(label);
			JScrollPane scrollPane = new JScrollPane(getSentence());
			scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			scrollPane.setPreferredSize(new Dimension(350, 250));
			sentencePanel.add(scrollPane);
			sentencePanel.add(getParseButton());
		}
		return sentencePanel;
	}

	/**
	 * @return the input
	 */
	private JTextArea getSentence() {
		if (sentence == null) {
			log.debug("create sentence");
			sentence = new JTextArea();
			sentence.setLineWrap(true);
			sentence.setWrapStyleWord(true);
			sentence.setFont(font);
		}
		return sentence;
	}

	/**
	 * @return
	 */
	private JButton getParseButton() {
		if (parseButton == null) {
			log.debug("create parse button");
			parseButton = new JButton("Analizuj");
			parseButton.setEnabled(false);
			parseButton.setActionCommand(COMMAND_PARSE);
			parseButton.addActionListener(this);
		}
		return parseButton;
	}

	/**
	 * @return
	 */
	private JPanel getLogPanel(){
		if (logPanel == null) {
			log.debug("create log panel");
			logPanel = new JPanel();
			logPanel.setLayout(new BoxLayout(logPanel, BoxLayout.Y_AXIS));
			JLabel label = new JLabel("Logi");
			label.setFont(font);
			logPanel.add(label);
			JScrollPane scrollPane = new JScrollPane(getLogText());
			scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			scrollPane.setPreferredSize(new Dimension(700, 150));
			logPanel.add(scrollPane);
		}
		return logPanel;
	}

	/**
	 * @return
	 */
	private JTextArea getLogText() {
		if (logText == null) {
			log.debug("create log");
			logText = new JTextArea();
			logText.setLineWrap(true);
			logText.setWrapStyleWord(true);
			logText.setEditable(false);
			logText.setFont(font);
		}
		return logText;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		log.debug("action performed");
		if (COMMAND_COMPILE.equals(e.getActionCommand())) {
			log.debug("compile grama");
			getLogText().append("---------------------------------------\n");
			log("Przetwarzanie gramatyki:");
			String grama = null;
			try {
				grama = getGrama().getText();
			} catch (NullPointerException e1) {
				log("Document is null (gramatyka) !!!", e1);
				return;
			}
			compileGrama(grama);
			if (syntax != null) {
				log.debug("disable compile");
				getCompileButton().setEnabled(false);
				getGrama().getDocument().addDocumentListener(this);
				log.debug("enable parse");
				getParseButton().setEnabled(true);
			}
		} else if (COMMAND_PARSE.equals(e.getActionCommand())) {
			log.debug("parse sentence");
			log("Przetwarzanie zdania:");
			String sentence = null;
			try {
				sentence = getSentence().getText();
			} catch (NullPointerException e1) {
				log("Document is null (zdanie)", e1);
				return;
			}
			parseSentence(sentence);
			//TODO:parse
			log.debug("disable parse");
			getParseButton().setEnabled(false);
			getSentence().getDocument().addDocumentListener(this);
		}
	}

	/**
	 * Zalogowanie komunikatu
	 * @param message
	 */
	public void log(String message) {
		log(message, null);
	}
	
	/**
	 * Zalogowanie komunikatu z towarzyszacym mu wyjatkiem
	 * @param message
	 */
	public void log(String message, Throwable t) {
		if (t == null) {
			log.debug(message);
		} else {
			log.warn(message, t);
		}
		StringBuffer msg = new StringBuffer();
		Calendar time = new GregorianCalendar();
		int tmp;
		tmp = time.get(Calendar.HOUR_OF_DAY);
		msg.append(tmp<10 ? "0"+tmp : tmp);
		msg.append(":");
		tmp = time.get(Calendar.MINUTE);
		msg.append(tmp<10 ? "0"+tmp : tmp);
		msg.append(":");
		tmp = time.get(Calendar.SECOND);
		msg.append(tmp<10 ? "0"+tmp : tmp);
		msg.append(",");
		tmp = time.get(Calendar.MILLISECOND);
		msg.append(tmp<100 ? (tmp<10 ? "00"+tmp : "0"+tmp) : tmp);
		msg.append(" - ");
		msg.append(message);
		if (t != null) {
			msg.append(" - ");
			msg.append(t.toString());
		}
		msg.append("\n");
		getLogText().append(msg.toString());
		Document d = getLogText().getDocument();
		getLogText().select(d.getLength(), d.getLength());
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
	 */
	public void changedUpdate(DocumentEvent e) {
		textChanged(e);
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
	 */
	public void insertUpdate(DocumentEvent e) {
		textChanged(e);
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
	 */
	public void removeUpdate(DocumentEvent e) {
		textChanged(e);
	}

	/**
	 * @param e
	 */
	private void textChanged(DocumentEvent e) {
		log.debug("text changed");
		if (e.getDocument() == getGrama().getDocument()){
			log.debug("grama changed");
			e.getDocument().removeDocumentListener(this);
			getCompileButton().setEnabled(true);
			log.debug("enabling compile");
			getParseButton().setEnabled(false);
			log.debug("disabling parse");
		} else if (e.getDocument() == getSentence().getDocument()) {
			log.debug("sentence changed");
			e.getDocument().removeDocumentListener(this);
			getParseButton().setEnabled(true);
			log.debug("enabling parse");
		}
	}
}
