/**
 * 
 */
package lm.parser.dynamic.syntax;

import java.util.HashMap;

import javax.management.InvalidAttributeValueException;

import lm.parser.InputString;
import lm.parser.ParseException;

import org.apache.log4j.Logger;

/**
 * @author Michal Jaros
 *
 */
public class Node {

	/**
	 * Logger
	 */
	private static final Logger log = Logger.getLogger(Node.class);

	/**
	 * Wezel dla symbolu pomocniczego
	 */
	public static final int NONTERMINAL = 0;

	/**
	 * Wezel dla symbolu koncowego
	 */
	public static final int TERMINAL = 1;

	/**
	 * Typ wezla
	 */
	private int type;

	/**
	 * Nazwa symbolu koncowego lub symbol terminalny
	 */
	private String token = null;

	/**
	 * Nastepny wezel
	 */
	private Node nextNode = null;

	/**
	 * Wezel alternatywny
	 */
	private Node alternativeNode = null;

	/**
	 * Konstruktor wezla
	 * @param type - typ wezla
	 * @param token - symbol koncowy lub pomocniczy
	 * @throws InvalidAttributeValueException 
	 */
	public Node(int type, String token) throws InvalidAttributeValueException{
		log.debug("Node " + type + " - " + token);
		switch (type) {
			case TERMINAL:
			case NONTERMINAL:
				this.type = type;
				break;
			default:
				throw new InvalidAttributeValueException("Invalid type of the node !!!");
		}
		if (token != null){
			this.token = token;
		} else {
			this.token = "";
		}
	}

	/**
	 * @return nastepny wezel
	 */
	public Node getNextNode() {
		return nextNode;
	}

	/**
	 * @param nextNode - nastepny wezel
	 */
	public void setNextNode(Node nextNode) {
		this.nextNode = nextNode;
	}

	/**
	 * @return wezel alternatywny
	 */
	public Node getAlternativeNode() {
		return alternativeNode;
	}

	/**
	 * @param alternativeNode - wezel alternatywny
	 */
	public void setAlternativeNode(Node alternativeNode) {
		this.alternativeNode = alternativeNode;
	}

	/**
	 * Zapobieganie nieskoczonym wywoaniom rekurencyjnym
	 */
	private int recursionCount = 0;

	/**
	 * @throws ParseException 
	 * @throws RecursionException 
	 * 
	 */
	public boolean parse(InputString input, HashMap<String, Node> syntax) throws ParseException, RecursionException{
		switch (type) {
			case TERMINAL:
				return input.check(token);
			case NONTERMINAL:
				recursionCount++;
				if (recursionCount > 1000){
					throw new RecursionException("Zbyt duo wywoa rekurencyjnych !!!");
				}
				try {
					Node curentNode = syntax.get(token);
					boolean firstNode = true;
					boolean error;
					do {
						error = !curentNode.parse(input, syntax);
						if (error){
							curentNode = curentNode.getAlternativeNode();
						}else{
							firstNode = false;
							curentNode = curentNode.getNextNode();
						}
					}while(curentNode != null);
					if (error && !firstNode){
						throw new ParseException("Bd skadniowy na pozycji " + (input.getCurentPosition()+1) + " !!!");
					}else{
						return !error;
					}
				} finally {
					recursionCount--;
				}
			default:
				log.error("Niewlasciwy typ wezla !!!");
				return false;
		}
	}
}
