/**
 * 
 */
package lm.parser.dynamic.syntax;

import java.util.ArrayList;

import org.apache.log4j.Logger;

/**
 * Drzewo (fragmet dynamicznej struktury danych zlozonej z conajmniej jednego wezla)
 * @author Michal Jaros
 */
public class Tree {

	/**
	 * Logger
	 */
	private static final Logger log = Logger.getLogger(Tree.class);

	/**
	 * Wezel wejsciowy dla danego drzewa
	 */
	private Node entryNode = null;

	/**
	 * Ostatni wezel alternatywny dla wezla wejsciowego
	 */
	private Node lastAlternalive = null;

	/**
	 * Zbior wezlow wyjsciowych
	 */
	private ArrayList<Node> endPoints = null;

	/**
	 * @param node
	 */
	public Tree(Node node){
		log.debug("New tree");
		entryNode = node;
		lastAlternalive = node;
		endPoints = new ArrayList<Node>();
		endPoints.add(node);
	}

	/**
	 * Dodanie nastepnego wezla
	 * @param node
	 */
	public void addNext(Node node){
		for (int i=0; i<endPoints.size(); i++) {
			Node endNode = endPoints.get(i);
			endNode.setNextNode(node);
		}
		endPoints.clear();
		endPoints.add(node);
	}

	/**
	 * Dodanie nastepnych wezlow
	 * @param tree
	 */
	public void addNext(Tree tree){
		for (int i=0; i<endPoints.size(); i++) {
			Node endNode = endPoints.get(i);
			endNode.setNextNode(tree.getEntryNode());
		}
		endPoints = tree.getEndPoints();
	}

	/**
	 * Dodanie alternatywnego wezla
	 * @param node
	 */
	public void addAlternative(Node node){
		lastAlternalive.setAlternativeNode(node);
		lastAlternalive = node;
		endPoints.add(node);
	}

	/**
	 * Dodanie alternatywy do drzewa
	 * @param tree - alternatywne drzewo
	 */
	public void addAlternative(Tree tree){
		lastAlternalive.setAlternativeNode(tree.getEntryNode());
		lastAlternalive = tree.getEntryNode();
		endPoints.addAll(tree.getEndPoints());
	}

	/**
	 * @return the entryNode
	 */
	public Node getEntryNode() {
		return entryNode;
	}

	/**
	 * @return the lastAlternalive
	 */
	public Node getLastAlternalive() {
		return lastAlternalive;
	}

	/**
	 * @return the endPoints
	 */
	private ArrayList<Node> getEndPoints() {
		return endPoints;
	}
}
